#!/bin/sh

[ -n "$include_widgets" ] && return 0
include_widgets="$0"

. "$_EXEC/cgilite/storage.sh"

w_refuri="$(URL "$PATH_INFO")?$(HTML "$QUERY_STRING")"

w_tags="$(find "$_DATA/" -path '*/.index/meta' -print0 \
        | xargs -r0 sed -r '
          s;^.*\t.*\t.*\ttags=(.*)\tcomment=.*\t.*\r$;\1;;
          s;,;\n;g;'"$UNSTRING" \
        | sort -u |HTML |sed 's;&#58\;;:;g; s;&#10\;;\n;g;')"
w_tagcategories="$(printf %s "$w_tags" | cut -sd: -f1 | uniq)"

[ "$ORDER" = Name   ] && w_coname=checked
[ "$ORDER" = Date   ] && w_codate=checked
[ "$ORDER" = Length ] && w_colength=checked

w_bookmarks(){
  local name='' cf='' cs='' bm="$_DATA/.index/bookmarks" proposed_name=''
  [ ! -d "${bm%/*}" ] && return 0
  [ ! -f "$bm" ] && touch "$bm"

  [ "$SEARCH" ] && cs="$(STRING "$SEARCH")"
  [ "$FILTER" ] && cf="$(STRING "$FILTER")"

  name="$(grep -m1 -aF "	search=$cs	filter=$cf${CR}" "$bm" |cut -f1 |UNSTRING)"
  [ ! "$name" ] && proposed_name="$(
    printf '%s\t%s' "$SEARCH" "$FILTER" \
    |sed -r '/^\t$/{  s;\t;All;; q;}
             /.*\t$/{ s;\t$;;; q;}
             /^\t.*/{ s;^\t;;;
                      :x; s;(^|[~^|])([^|^~:]+):;\1;; tx;
                      s;\^; and ;g; s;\|;,;g; s;~;not ;g; q;}'
    )"

  printf '[form #bookmarks action=?a=bookmark method=POST
            [a href="#" x]
            [hidden "ref" "%s"]
            [hidden "search" "%s"][hidden "filter" "%s"]
            [label Name for current page:]
            [input name="name" value="%s" placeholder="Name" ]
            [button type="submit" %s]' \
            "$w_refuri" \
            "$(HTML "$SEARCH")" "$(HTML "$FILTER")" \
            "$(HTML "${name:-${proposed_name}}")" \
            "${name:+Update}${proposed_name:+Add}"
  [ "$name" ] && printf ' [submit "delete" "delete" Delete]'

  sort "$bm" |while read -r name search filter; do
    search="${search#search=}" filter="${filter#filter=}" filter="${filter%${CR}}"
    [ "$search" = "${cs}" -a "$filter" = "${cf}" ] && continue

    name="$(UNSTRING "$name")";
    search="$(UNSTRING "${search}" |URL)";
    filter="$(UNSTRING "${filter}" |URL)";
    printf '[label .link %s]
            [a .link target=blank href="?o=Name&s=%s&f=%s" by Name]
            [a .link target=blank href="?o=Date&s=%s&f=%s" by Date]
            [a .link target=blank href="?o=Length&s=%s&f=%s" by Length]
            [br]' \
            "$(HTML "$name")" \
            "$search" "$filter" \
            "$search" "$filter" \
            "$search" "$filter"
  done
  printf ']'
}

w_search(){
  printf '
  [form #search method=GET action=?
    [select name=o size=1 
      [option disabled=disabled Order By]
      [option value=Name %s Name]
      [option value=Date %s Date]
      [option value=Length %s Length]
    ]
    [input name=s placeholder=Search value="%s"]
  ]
  ' \
  "$w_coname" "$w_codate" "$w_colength" \
  "$(HTML "$SEARCH")"
}

w_prefs(){
  local tm tf

  tm=''; [ "$(COOKIE mode)" = index ] && tm=' '
  tf=''; [ "$(COOKIE fakemp4)" = yes ] && tf=checked

  printf '
  [form #prefs method="POST" action="?a=setprefs"
    [a href="#" x]
    [hidden "ref" "%s"]
    [label for=prefs_ps Pagesize]
    [input #prefs_ps type=number name=pagesize value="%s"][br]
    [radio "mode" "browse" %s #prefs_modebrowse] [label for=prefs_modebrowse Browse Folders][br]
    [radio "mode" "index"  %s #prefs_modeindex ] [label for=prefs_modeindex View Full Index][br]
    [checkbox "fakemp4" "yes" %s #prefs_fmp4] [label for=prefs_fmp4 Fake .MP4 file ending][br]
    [submit "store" "store" Set Cookie]
  ]
  ' \
  "$w_refuri" "$LISTSIZE" \
  "${tm:-checked}" "${tm:+checked}" "$tf"
}

w_index(){
  [ -d "$_DATA/$ITEM/.index" ] || printf '
  [form #index method="POST" action="?a=spawnindex"
    [hidden "ref" "%s"]
    [label Set up for Index view: ]
    [checkbox "recursive" "yes" #spawn_recursive] [label for=spawn_recursive Include subdirectories]
    [submit "spawn" "spawn" Set up]
  ]
  ' "$w_refuri"
}

w_advsearch(){
  local n lbid tag category filter f t
  filter="$(HTML "${FILTER}^" |sed 's;&#126\;;~;g; s;&#94\;;^;g; s;&#124\;;|;g; s;&#58\;;:;g;')"

  printf '[form #advsearch action=?a=advsearch method=POST
            [a href="#" Hide]
            [p .help Select multiple tags from each category by holding down the [strong Ctrl] key on your keyboard.[br]
            Refine the search further by setting additional search tags using the [strong "+and"] button.]'

  for n in 1 2 3 4 5 6 7 8 9 10; do
    f="${filter%%^*}"; filter="${filter#*^}"

    t=''; [ "$f" -a ! "${f%%~*}" ] && t=" "

    lbid="cat_${n}_(none)"
    printf '[input .and type=checkbox name=and id="and_%i" %s][label for="and_%i" +and
            ][fieldset .select
            [radio "pol_%i" "pos" .pol %s #pol_pos_%i"][label for=pol_pos_%i Any]
            [radio "pol_%i" "neg" .pol %s #pol_neg_%i"][label for=pol_neg_%i None]
            [label .head Category:]' \
            $n "${f:+checked}" $n \
            $n "${t:-checked}" $n $n \
            $n "${t:+checked}" $n $n

    f="|${f#~}|"
    printf '*\n%s\n' "$w_tagcategories" \
    | while read -r category; do
      lbid="cat_${n}_${category}"

      t=''
      [ "$category"  = '*' -a   "${f%%|${category}:*}" ] && t=checked
      [ "$category" != '*' -a ! "${f%%|${category}:*}" ] && t=checked

      printf '[radio "cat_%i" "%s" .cat %s id="%s"][label for="%s" %s]
              [select name=tag_%s size=10 multiple' \
              $n "$category" "$t" "$lbid" "$lbid" "$category" $n

      printf '%s\n' "$w_tags" \
      | { [ "$category" = '*' ] && grep -avF ':' |grep -avxF '' || grep -awF "${category}"; } \
      | while read -r tag; do
        t=''; [ ! "${f%%*|${tag}|*}" ] && t=checked
        printf '[option %s value="%s"\n%s]' "$t" "$tag" "${tag#*:}"
      done
      printf '\n]'
    done
    printf ']'
  done

  printf '[fieldset .submit [select name=order
            [option disabled=disabled Order By]
            [option value=Name %s Name]
            [option value=Date %s Date]
            [option value=Length %s Length]
          ][button type=submit Apply Filter]]
          ]' \
          "$w_coname" \
          "$w_codate" \
          "$w_colength"
}

w_tagging(){
  local tag category
  printf '[div #multitag [input type="hidden" name="ref" value="%s"]' "$w_refuri"
  printf '[a href="#" Hide][br]'

  printf 'Tags\n%s\n' "$w_tagcategories" \
  | while read -r category; do
    printf '[fieldset [legend %s:][select name=tag size=4 multiple' "$category"
    printf %s "$w_tags" \
    | { [ "$category" = 'Tags' ] && grep -avF ':' | grep -avxF '' || grep -awF "${category}"; } \
    | while read -r tag; do
      printf '[option value="%s"\n%s]' "$tag" "${tag#*:}"
    done
    printf ']]'
  done

  printf '[fieldset [legend New:][textarea name=newtag\n][button type=Submit Add Tags]]]'
}
